;; Guitar to Floppy interface
;;
;; Copyright 2012 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike to mikekohn.net
;;
;; Take input from a guitar and vibrate the head of a floppy disk
;; based on the input.

.include "msp430x2xx.inc"

  org 0xf800
start:
  ;; Turn off watchdog *WOOF*
  mov.w #0x5a80, &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; Set interrupt routine
  mov.w #init_delay, r13
  mov.w #1000, r12
  mov.w #0, r5

  ;; Set up stack pointer
  mov.w #0x0280, SP

  ;; Set up crystal
  mov.b #(DCO_3), &DCOCTL
  mov.b #(RSEL_15), &BCSCTL1
  mov.b #0, &BCSCTL2

  ;; Set up output pins
  ;; P1.0 = step
  ;; P1.1 = direction
  ;; P1.2 = select A
  ;; P1.3 = motor enable A
  ;; P1.5 = debug LED
  ;; P1.6 = track 0
  mov.b #0x2f, &P1DIR        ; P1.0 P1.1
  ;mov.b #0x04, &P1SEL        ; for A/D on pin A2
  mov.b #0x2c, &P1OUT        ; P1.0=/CS

  ;; Set up SPI
  ;mov.b #(USIPE6|USIPE5|USIMST|USIOE|USISWRST), &USICTL0
  ;mov.b #USICKPH, &USICTL1
  ;mov.b #(USIDIV_0|USISSEL_2), &USICKCTL ; div 1, SMCLK
  ;bic.b #USISWRST, &USICTL0      ; clear reset

  ;; Set up A/D
  mov.w #(ADC10ON), &ADC10CTL0
  mov.w #(INCH_7|ADC10SSEL_2), &ADC10CTL1

  ;; Set up Timer
  mov.w #32000, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  ;; Okay, I can be interrupted now
  eint

main:
  bit.b #0x40, &P1IN
  jz led_off
  bis.b #0x20, &P1OUT
  jmp main
led_off:
  bic.b #0x20, &P1OUT
  jmp main

timer_interrupt:
  br r13

read_guitar:
  mov.w &ADC10MEM, r9

  cmp.w #(512+12), r9
  jge guitar_is_high
  cmp.w #(512-12), r9
  jge guitar_is_low
leave:
  bis.w #(ENC|ADC10SC), &ADC10CTL0
  bit.b #0x40, &P1IN
  jnz not_track_80
  mov.w #80, r12
  mov.w #move_to_forty, r13
  bis.b #2, &P1OUT
  bic.b #1, &P1OUT
  mov.w #32000, &TACCR0
not_track_80:
  reti
guitar_is_high:
  cmp.b #1, r5
  jeq leave
  mov.b #1, r5
  xor.b #1, &P1OUT
  jmp leave
guitar_is_low:
  cmp.b #0, r5
  jeq leave
  mov.b #0, r5
  xor.b #1, &P1OUT
  xor.b #2, &P1OUT
  jmp leave

init_delay:
  dec.w r12
  jz init_delay_done
  reti
init_delay_done:
  mov #move_to_zero, r13
  mov.w #160, r12
  bic.b #2, &P1OUT
  reti

move_to_zero:
  xor.b #1, &P1OUT
  bit.b #1, &P1OUT
  jnz dont_check
  bit.b #0x40, &P1IN
  ;jz move_to_zero_done
dont_check:
  dec.w r12
  jz move_to_zero_done
  reti
move_to_zero_done:
  mov.w #move_to_forty, r13
  mov.w #80, r12
  bis.b #2, &P1OUT
  reti

move_to_forty:
  xor.b #1, &P1OUT
  bit.b #1, &P1OUT
  dec.w r12
  jz move_to_forty_done
  reti
move_to_forty_done:
  mov.w #500, &TACCR0
  mov.w #read_guitar, r13
  reti


  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset


